==> iosxr-eznc-2016.7.27dev.tar.gz: iosxr-eznc-2016.7.27dev/iosxr_eznc/decorators.py <==
# -*- coding: utf-8 -*-
# Copyright 2016 CloudFlare, Inc. All rights reserved.
#
# The contents of this file are licensed under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with the
# License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

"""
Useful decorators.
"""


from __future__ import absolute_import

# import stdlib
import inspect
from functools import wraps

# import third party
from lxml import etree
from ncclient.transport.errors import TransportError as NcTpError
from ncclient.operations.errors import TimeoutExpiredError as NcTEError
from ncclient.operations.rpc import RPCError as NcRPCError

# import local modules
from iosxr_eznc.exception import RPCError as _XRRPCError
from iosxr_eznc.exception import ConnectionClosedError
from iosxr_eznc.exception import RPCTimeoutError



def raise_eznc_exception(fun):

    def _get_rpc_error_class(exc):

        mod = __import__('iosxr_eznc.exception')
        for obj in inspect.getmembers(mod.exception):
            # digging into exceptions' module
            if obj[0] == exc and inspect.isclass(obj):
                # found ya
                return obj

    @wraps(fun)
    def _raise_eznc_exception(*vargs, **kvargs):
        ### ~~~ vargs[0] is dev obj ~~~
        try:
            return fun(*vargs, **kvargs)
        except NcRPCError as nc_err:
            exc = '{}Error'.format(fun.__name__.title().replace('_', ''))
            XRRPCError = _get_rpc_error_class(exc)
            if not XRRPCError:
                # could not find the desired exception, throw the base
                XRRPCError = _XRRPCError
            if msg:
                nc_err.args[0]['msg'] = msg
            raise XRRPCError(vargs[0], nc_err.args[0])
        except NcTEError as nc_err:
            err = {
                'fun': fun.__name__,
                'timeout': vargs[0].timeout
            }
            raise RPCTimeoutError(vargs[0], err)
        except NcTpError as nc_err:
            raise ConnectionClosedError(vargs[0])

    return _raise_eznc_exception


def qualify(param, oper=None):

    def _qualify_wrapper(fun):

        @wraps(fun)
        def _qualify(*vargs, **kvargs):
            if param in kvargs.keys():
                xml_req_tree = kvargs[param]
                if isinstance(xml_req_tree, basestring):
                    xml_req_tree = etree.fromstring(xml_req_tree)
                if xml_req_tree.get('xmlns') is None:
                    namespace = vargs[0]._dev._namespaces.get(xml_req_tree.tag, oper=oper)
                    if not namespace is None:
                        # cannot register with namespace
                        # probably the request will fail...
                        xml_req_tree.set('xmlns', namespace)
                        kvargs[param] = xml_req_tree
            return fun(*vargs, **kvargs)

        return _qualify

    return _qualify_wrapper

==> iosxr-eznc-2016.8.4.tar.gz: iosxr-eznc-2016.8.4/iosxr_eznc/decorators.py <==
# -*- coding: utf-8 -*-
# Copyright 2016 CloudFlare, Inc. All rights reserved.
#
# The contents of this file are licensed under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with the
# License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

"""
Useful decorators.
"""


from __future__ import absolute_import

# import stdlib
import inspect
from functools import wraps

# import third party
from lxml import etree
from ncclient.transport.errors import TransportError as NcTpError
from ncclient.operations.errors import TimeoutExpiredError as NcTEError
from ncclient.operations.rpc import RPCError as NcRPCError

# import local modules
from iosxr_eznc.exception import RPCError as _XRRPCError
from iosxr_eznc.exception import ConnectionClosedError
from iosxr_eznc.exception import RPCTimeoutError


def raise_eznc_exception(fun):

    def _get_rpc_error_class(exc):

        mod = __import__('iosxr_eznc.exception')
        for obj in inspect.getmembers(mod.exception):
            # digging into exceptions' module
            if obj[0] == exc and inspect.isclass(obj):
                # found ya
                return obj

    @wraps(fun)
    def _raise_eznc_exception(*vargs, **kvargs):
        ### ~~~ vargs[0] is rpc obj ~~~
        _rpc_obj = vargs[0]
        _dev_obj = _rpc_obj._dev
        try:
            return fun(*vargs, **kvargs)
        except NcRPCError as nc_err:
            exc = '{}Error'.format(fun.__name__.title().replace('_', ''))
            XRRPCError = _get_rpc_error_class(exc)
            if not XRRPCError:
                # could not find the desired exception, throw the base
                XRRPCError = _XRRPCError
            err = nc_err.args[0]
            err.update({
                'fun': 'rpc.{}'.format(fun.__name__),
                'args': vargs[1:] if len(vargs) > 0 else [],
                'kvargs': kvargs
            })
            raise XRRPCError(_dev_obj, err)
        except NcTEError as nc_err:
            err = {
                'fun': fun.__name__,
                'timeout': _dev_obj.timeout
            }
            raise RPCTimeoutError(_dev_obj, err)
        except NcTpError as nc_err:
            raise ConnectionClosedError(_dev_obj)

    return _raise_eznc_exception


def qualify(param, oper=None):

    def _qualify_wrapper(fun):

        @wraps(fun)
        def _qualify(*vargs, **kvargs):
            if param in kvargs.keys():
                xml_req_tree = kvargs[param]
                if isinstance(xml_req_tree, basestring):
                    xml_req_tree = etree.fromstring(xml_req_tree)
                if xml_req_tree.get('xmlns') is None:
                    namespace = vargs[0]._dev._namespaces.get(xml_req_tree.tag, oper=oper)
                    if not namespace is None:
                        # cannot register with namespace
                        # probably the request will fail...
                        xml_req_tree.set('xmlns', namespace)
                        kvargs[param] = xml_req_tree

            return fun(*vargs, **kvargs)

        return _qualify

    return _qualify_wrapper


def wrap_xml(param, tag='filter'):

    def _wrap_xml_wrapper(fun):

        @wraps(fun)
        def _wrap_xml(*vargs, **kvargs):
            if param in kvargs.keys():
                xml_req_tree = kvargs[param]
                if isinstance(xml_req_tree, basestring):
                    xml_req_tree = etree.fromstring(xml_req_tree)
                if xml_req_tree.tag != tag:
                    tag_elem = etree.Element(tag)
                    tag_elem.append(xml_req_tree)
                    kvargs[param] = tag_elem

            return fun(*vargs, **kvargs)

        return _wrap_xml

    return _wrap_xml_wrapper
